
/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2001 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      Inc. and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

DEF VAR Queue_i_RCSVersion AS CHARACTER INIT "$Header: /home/sauge/code/progress/ds/queue/queue.i,v 1.2 2001/11/17 06:35:22 sauge Exp sauge $" NO-UNDO.

/*****************************************************************/
/* All functions return a logical value.  If the logical value   */
/* is TRUE, then an error occurred.  Use the pError value to     */
/* better understand the kind of error that occurred.            */
/*****************************************************************/

/*****************************************************************/
/*                  Is the named queue empty?                    */
/* pName - Name of the queue, as all data goes into a given      */
/*         table.                                                */
/* pID   - Item queueed.  This is a unique reference to a record */
/*         in a different table.  The queueing algs do not       */
/*         handle data, only the organization of the data.       */
/* RETURNS CHAR                                                  */
/*  1    No Name                                                 */
/*  2    Is Empty                                                */
/*  3    Has Contents                                            */
/*****************************************************************/

FUNCTION QueEmpty RETURNS LOGICAL
(INPUT pName   AS CHARACTER,
 OUTPUT pError AS CHARACTER):

  IF pName = ? OR pName = "" THEN DO:
    ASSIGN pError = "1".
    RETURN TRUE.
  END.

  IF NOT CAN-FIND (FIRST DS WHERE DS.Name = pName) THEN ASSIGN pError = "2".
  ELSE ASSIGN pError = "3".

  RETURN FALSE.

END. /* FUNCTION QueEmpty */

/*****************************************************************/
/*                  Enque an entry on the queue                  */
/* pName - Name of the queue, as all data goes into a given      */
/*         table.                                                */
/* pID   - Item queueed.  This is a unique reference to a record */
/*         in a different table.  The queueing algs do not       */
/*         handle data, only the organization of the data.       */
/* pError - When returning an error, use this to get a more des- */
/*          criptive problem.                                    */
/* RETURNS CHAR                                                  */
/*  0    Successful                                              */
/*  1    No Name                                                 */
/*  2    No ID to data record                                    */
/*****************************************************************/

FUNCTION QueEnque RETURNS LOGICAL
(INPUT pName   AS CHARACTER,
 INPUT pID     AS CHARACTER,
 OUTPUT pError AS CHARACTER): 

  DEF VAR cNewDSID  AS CHARACTER NO-UNDO.
  DEF VAR cPrevDSID AS CHARACTER NO-UNDO.

  /* Look for argument errors */

  IF pName = ? OR pName = "" THEN DO:
    ASSIGN pError = "1".
    RETURN TRUE.
  END.

  IF pID = ? OR pID = "" THEN DO:
    ASSIGN pError = "2".
    RETURN TRUE.
  END.   

  /* Create a record to tag at the top of the queue */

  ASSIGN cNewDSID = MakeID2(10).

  /* If there is entries, look up the LAST entry and     */
  /* set it's NextDSID = to the new DSID we are creating */

  FIND DS EXCLUSIVE-LOCK
  WHERE DS.NextDSID = ?
  NO-ERROR.

  IF AVAILABLE DS THEN 
    ASSIGN 
    DS.NextDSID = cNewDSID
    cPrevDSID = DS.DSID.
  ELSE 
    ASSIGN
    cPrevDSID = ?.

  /* Go about creating the new record. */

  CREATE DS.

  ASSIGN 
  DS.DSID = cNewDSID
  DS.Name = pName
  DS.DataID = pID
  DS.NextDSID = ?
  DS.PrevDSID = cPrevDSID.

  RETURN FALSE.

END. /* FUNCTION QueEnque */

/*****************************************************************/
/*              Dequeue an entry off the queue                   */
/* pName - Name of the queue, as all data goes into a given      */
/*         table.                                                */
/* pID   - Item queueed.  This is a unique reference to a record */
/*         in a different table.  The queueing algs do not       */
/*         handle data, only the organization of the data.       */
/* RETURNS CHAR                                                  */
/*  0    Successful                                              */
/*  1    No Name                                                 */
/*  2    queue is empty                                          */
/*****************************************************************/

FUNCTION QueDeque RETURNS LOGICAL
(INPUT pName   AS CHARACTER,
 OUTPUT pID    AS CHARACTER,
 OUTPUT pError AS CHARACTER):

  DEF VAR cNextDSID  AS CHARACTER NO-UNDO.

  /* Determine if we have an argument error */

  IF pName = ? OR pName = "" THEN DO:
    ASSIGN pError = "1".
    RETURN TRUE.
  END.

  /* Determine if we are trying to pop an empty queue */

  QueEmpty(INPUT pName, OUTPUT pError).
  IF pError = "2" THEN RETURN TRUE.

  /* Pull a value off the queue.  The first item has PrevDSID = ?. */

  FIND DS EXCLUSIVE-LOCK
  WHERE DS.PrevDSID = ?
  NO-ERROR.

  ASSIGN 
  pID = DS.DataID.

  /* We have the data link in pID.  Now we want to remove */
  /* the entry.  We save the DSID of the entry before the */
  /* current one so we can set it's NextDSID = ? (after   */
  /* all we are trashing this record) and delete the cur- */
  /* rent record.                                         */

  ASSIGN 
  cNextDSID = DS.NextDSID.

  DELETE DS.

  FIND DS EXCLUSIVE-LOCK
  WHERE DS.DSID = cNextDSID
  NO-ERROR.

  IF AVAILABLE DS THEN
    ASSIGN DS.PrevDSID = ?.

  RETURN FALSE.

END. /* FUNCTION QueDequeue */


/*****************************************************************/
/*                      Clear the queue                          */
/* pName - Name of the queue, as all data goes into a given      */
/*         table.                                                */
/* RETURNS CHAR                                                  */
/*  0    Successful                                              */
/*  1    No Name                                                 */
/*****************************************************************/

FUNCTION QueClear RETURNS LOGICAL
(INPUT pName   AS CHARACTER,
 OUTPUT pError AS CHARACTER):

  IF pName = ? OR pName = "" THEN DO:
    ASSIGN pError = "1".
    RETURN TRUE.
  END.

  FOR EACH DS EXCLUSIVE-LOCK
  WHERE DS.Name = pName:
     DELETE DS.
  END.

  ASSIGN pError = "0".
  RETURN FALSE.

END. /* FUNCTION QueClear */

/*****************************************************************/
/*        Read an item from the queue without removing           */
/* pName - Name of the stack, as all data goes into a given      */
/*         table.                                                */
/* RETURNS CHAR                                                  */
/*  0    Successful                                              */
/*  1    No Name                                                 */
/*  2    Empty                                                   */
/*****************************************************************/

FUNCTION QueRead RETURNS LOGICAL
(INPUT pName   AS CHARACTER,
 OUTPUT pID    AS CHARACTER,
 OUTPUT pError AS CHARACTER):

  /* Look for argument errors */

  IF pName = ? OR pName = "" THEN DO:
    ASSIGN pError = "1".
    RETURN TRUE.
  END.

  /* Determine if the stack is empty.  If so, we need to bail */
  QueEmpty (INPUT pName, pError).
  IF pError = "2" THEN RETURN TRUE.

  /* Here we find the record and merely read the value.  */

  FIND DS NO-LOCK
  WHERE DS.Name = pName
  AND DS.PrevDSID = ?
  NO-ERROR.

  ASSIGN pID = DS.DataID.

  ASSIGN pError = "0".
  RETURN FALSE.


END. /* FUNCTION QueRead */
